#import <objc/Object.h>

/* HTMLConverter bundle class
 * This class is associated with the Mail application.
 * It will convert HTML tags before displaying them in the Mail window.
 */
@interface HTMLConverter:Object
{
    /* conversion info */
    BOOL performHTMLConversion;
    const char *htmlMapFileName;
    NXStringTable *conversionMap;
    
    /* display info */
    BOOL performHTMLDisplay;
    const char *htmlSavePathName;
    const char *htmlServiceName;
    int fileCount;
}

/* Class Methods */
+ initialize;

/* Instance Methods */
- init;
- free;

/* Accessor Methods */
    /* conversion info */
- (BOOL)performHTMLConversion;
- (void)setPerformHTMLConversion:(BOOL)value;
- (const char *)htmlMapFileName;
- (void)setHtmlMapFileName:(const char *)value;
- (NXStringTable *)conversionMap;
- (void)setConversionMap:(NXStringTable *)value;
    /* display info */
- (BOOL)performHTMLDisplay;
- (void)setPerformHTMLDisplay:(BOOL)value;
- (const char *)htmlSavePathName;
- (void)setHtmlSavePathName:(const char *)value;
- (const char *)htmlServiceName;
- (void)setHtmlServiceName:(const char *)value;
- (int)fileCount;
- (void)setFileCount:(int)value;

- (void)loadDefaults;

/* methods to handle HTML display & conversion */
- (BOOL)convertText:(Text*)text;
- (BOOL)openBrowserWithText:(Text*)text;
- (BOOL)saveText:(Text*)text toFileNamed:(const char *)filename;


/* Delegate Methods */
- (void)willDisplayText:(Text *)text;

@end


/* The bundle's main Class must register itself with NXApp
 * using this method in its +initialize method.
 * Note: "-addDisplayFilter:" takes an instance as an argument (not a Class).
 */
@interface Application(MailDisplayBundle)
- (void)addDisplayFilter:filterObject;
@end
